/* vi:set et ai sw=2 sts=2 ts=2: */
/*-
 * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib-object.h>

#include <gst-thumb-flavor.h>

/* property identifiers */
enum
{
	PROP_0,
	PROP_NAME,
	PROP_WIDTH,
	PROP_HEIGHT,
	PROP_MIN_WIDTH,
	PROP_MIN_HEIGHT,
	PROP_FRAME_SELECTION,
	PROP_ROTATION
};

static void thumber_flavor_finalize     (GObject      *object);
static void thumber_flavor_get_property (GObject      *object,
                                         guint         prop_id,
                                         GValue       *value,
                                         GParamSpec   *pspec);
static void thumber_flavor_set_property (GObject      *object,
                                         guint         prop_id,
                                         const GValue *value,
                                         GParamSpec   *pspec);

struct _ThumberFlavorClass
{
	GObjectClass __parent__;
};

struct _ThumberFlavor
{
	GObject __parent__;

	gchar    *name;
	gint      width;
	gint      height;
	gint      min_width;
	gint      min_height;
	gchar    *frame_selection;
        gboolean  rotation;
};

G_DEFINE_TYPE (ThumberFlavor, thumber_flavor, G_TYPE_OBJECT)

static void
thumber_flavor_class_init (ThumberFlavorClass *klass)
{
	GObjectClass *gobject_class;

	/* Determine the parent type class */
	thumber_flavor_parent_class = g_type_class_peek_parent (klass);

	gobject_class = G_OBJECT_CLASS (klass);
	gobject_class->finalize = thumber_flavor_finalize;
	gobject_class->get_property = thumber_flavor_get_property;
	gobject_class->set_property = thumber_flavor_set_property;

	g_object_class_install_property (gobject_class, PROP_NAME,
	                                 g_param_spec_string ("name",
	                                                      "name",
	                                                      "name",
	                                                      NULL,
	                                                      G_PARAM_READWRITE |
	                                                      G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_WIDTH,
	                                 g_param_spec_int ("width",
	                                                   "width",
	                                                   "width",
	                                                   -1, G_MAXINT, 0,
	                                                   G_PARAM_READWRITE |
	                                                   G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_HEIGHT,
	                                 g_param_spec_int ("height",
	                                                   "height",
	                                                   "height",
	                                                   -1, G_MAXINT, 0,
	                                                   G_PARAM_READWRITE |
	                                                   G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_MIN_WIDTH,
	                                 g_param_spec_int ("min_width",
	                                                   "min_width",
	                                                   "min_width",
	                                                   -1, G_MAXINT, 0,
	                                                   G_PARAM_READWRITE |
	                                                   G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_MIN_HEIGHT,
	                                 g_param_spec_int ("min_height",
	                                                   "min_height",
	                                                   "min_height",
	                                                   -1, G_MAXINT, 0,
	                                                   G_PARAM_READWRITE |
	                                                   G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_FRAME_SELECTION,
	                                 g_param_spec_string ("frame_selection",
	                                                      "frame_selection",
	                                                      "frame_selection",
	                                                      "Normal",
	                                                      G_PARAM_READWRITE |
	                                                      G_PARAM_CONSTRUCT_ONLY));

	g_object_class_install_property (gobject_class, PROP_ROTATION,
	                                 g_param_spec_string ("rotation",
							      "rotation",
							      "rotation",
							      "Rotate",
							      G_PARAM_READWRITE |
							      G_PARAM_CONSTRUCT_ONLY));
}

static void
thumber_flavor_init (ThumberFlavor *flavor)
{
}

static void
thumber_flavor_finalize (GObject *object)
{
	ThumberFlavor *flavor = THUMBER_FLAVOR (object);

	g_free (flavor->name);

	(*G_OBJECT_CLASS (thumber_flavor_parent_class)->finalize) (object);
}

static void
thumber_flavor_get_property (GObject    *object,
                             guint       prop_id,
                             GValue     *value,
                             GParamSpec *pspec)
{
	ThumberFlavor *flavor = THUMBER_FLAVOR (object);

	switch (prop_id) {
		case PROP_NAME:
			g_value_set_string (value, flavor->name);
			break;
		case PROP_WIDTH:
			g_value_set_int (value, flavor->width);
			break;
		case PROP_HEIGHT:
			g_value_set_int (value, flavor->height);
			break;
		case PROP_MIN_WIDTH:
			g_value_set_int (value, flavor->min_width);
			break;
		case PROP_MIN_HEIGHT:
			g_value_set_int (value, flavor->min_height);
			break;
		case PROP_FRAME_SELECTION:
			g_value_set_string (value, flavor->frame_selection);
			break;
		case PROP_ROTATION:
			g_value_set_string (value, flavor->rotation);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
thumber_flavor_set_property (GObject      *object,
                             guint         prop_id,
                             const GValue *value,
                             GParamSpec   *pspec)
{
	ThumberFlavor *flavor = THUMBER_FLAVOR (object);

	switch (prop_id) {
		case PROP_NAME:
			flavor->name = g_value_dup_string (value);
			break;
		case PROP_WIDTH:
			flavor->width = g_value_get_int (value);
			break;
		case PROP_HEIGHT:
			flavor->height = g_value_get_int (value);
			break;
		case PROP_MIN_WIDTH:
			flavor->min_width = g_value_get_int (value);
			break;
		case PROP_MIN_HEIGHT:
			flavor->min_height = g_value_get_int (value);
			break;
		case PROP_FRAME_SELECTION:
			flavor->frame_selection = g_value_dup_string(value);
			break;
		case PROP_ROTATION:
			flavor->rotation = g_value_dup_string(value);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

ThumberFlavor *
thumber_flavor_new (const gchar *name,
                    gint         width,
                    gint         height,
                    gint         min_width,
                    gint         min_height,
                    const gchar *frame_selection,
		    const gchar *rotation)
{
	g_return_val_if_fail (name != NULL && *name != '\0', NULL);

	return g_object_new (THUMBER_TYPE_FLAVOR,
	                     "name", name,
	                     "width", width,
	                     "height", height,
	                     "min_width", min_width,
	                     "min_height", min_height,
	                     "frame_selection", frame_selection,
			     "rotation", rotation,
	                     NULL);
}

ThumberFlavor *
thumber_flavor_new_normal (void)
{
	return g_object_new (THUMBER_TYPE_FLAVOR,
	                     "name", "normal",
	                     "width", 128,
	                     "height", 128,
	                     "frame_selection", "normal",
			     "rotation", "Rotate",
	                     NULL);
}

ThumberFlavor *
thumber_flavor_new_large (void)
{
	return g_object_new (THUMBER_TYPE_FLAVOR,
	                     "name", "large",
	                     "width", 256,
	                     "height", 256,
	                     "frame_selection", "normal",
			     "rotation", "Rotate",
	                     NULL);
}

const gchar *
thumber_flavor_get_name (const ThumberFlavor *flavor)
{
	g_return_val_if_fail (THUMBER_IS_FLAVOR (flavor), NULL);
	return flavor->name;
}

void
thumber_flavor_get_size (const ThumberFlavor    *flavor,
                         gint                   *width,
                         gint                   *height,
                         gint                   *min_width,
                         gint                   *min_height)
{
	g_return_if_fail (THUMBER_IS_FLAVOR (flavor));

	if (width != NULL)
		*width = flavor->width;

	if (height != NULL)
		*height = flavor->height;

	if (min_width != NULL)
		*min_width = flavor->min_width;

	if (min_height != NULL)
		*min_height = flavor->min_height;
}

const gchar *
thumber_flavor_get_frame_selection (const ThumberFlavor *flavor)
{
	g_return_val_if_fail (THUMBER_IS_FLAVOR (flavor), NULL);
	return flavor->frame_selection;
}

const gchar *
thumber_flavor_get_rotation (const ThumberFlavor *flavor)
{
	g_return_val_if_fail (THUMBER_IS_FLAVOR (flavor), NULL);
	return flavor->rotation;
}
